package com.tzld.piaoquan.recommend.server.service.rank.processor;

import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.util.MathUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author sunxy
 */
public class RankProcessorInsert {

    public static void insertByTag(RankParam param, List<Video> rovList,
                                   Map<String, Map<String, String>> rulesMap) {
        if (CollectionUtils.isEmpty(rovList) || rulesMap == null || rulesMap.isEmpty()) {
            return;
        }
        String insertRules = null, tagName = null;
        for (Map.Entry<String, Map<String, String>> entry : rulesMap.entrySet()) {
            Map<String, String> value = entry.getValue();
            if (value.containsKey("insert")) {
                insertRules = value.get("insert");
                tagName = entry.getKey();
                break;
            }
        }
        if (StringUtils.isBlank(insertRules) || StringUtils.isBlank(tagName)) {
            return;
        }
        final String finalTagName = tagName;

        // 判断是否前几个已经有该tag的视频了
        boolean isExistResultTagVideo = rovList.stream()
                .limit(param.getSize())
                .anyMatch(video -> video.getTags().contains(finalTagName));
        if (isExistResultTagVideo) {
            return;
        }

        // 获取需要插入的视频
        Video insertTagVideo = rovList.stream()
                .filter(video -> video.getTags().contains(finalTagName))
                .findFirst().orElse(null);
        if (insertTagVideo == null) {
            return;
        }

        // 获取插入的权重
        String[] insertWeight = insertRules.split(",");
        List<Double> insertWeightList = Arrays.stream(insertWeight)
                .map(Double::valueOf).collect(Collectors.toList());

        // 遍历权重，随机插入
        for (int i = 0; i < insertWeightList.size(); i++) {
            double probability = insertWeightList.get(i);
            double randomDouble = MathUtil.nextDouble(0, 1);
            if (randomDouble <= probability) {
                rovList.add(i, insertTagVideo);
                break;
            }
        }

    }

}
